/*LFMKV.G :  This proc evaluates the Markov switching likelihood function

    Global variable meanings:
        y = data to be analyzed
        ix = 0 to make innovation variance independent of the state
           = 1 to have different variances for each state
        iy = 0 means that the probability that the first observation is
             drawn from state 1 (rho) is related to the Markov transition
             probabilities (p and q) by the formula rho = (1-q)/(2-p-q)
           = 1 means rho is treated as a free parameter
           = 2 means rho is fixed at 1
           = 3 means rho is fixed at 0
        sameness = 0 means that explanitory variables differ for all 4 eqn's
                 = 1 means that same variables used for means in states 1&2
                 = 2 means that same variables used for probs in states 1&2
                 = 3 is the combination of sameness=1 and sameness=2
                 = 4 means that same variables used for all 4 equations
        vei[i] = number of parameters in equation i
	  useDLL = binary variable, true means use external C code

The order in which parameters are passed is:
    { alpha1, alpha2, pth, qth, sigma1, [sigma2,] [rho] }
where
    alpha1 = parameters for mean in state 1
    alpha2 = parameters for mean in state 2
    cdfn(x3*pth) = prob. of going from state 1 to state 1
    cdfn(x4*qth) = prob. of going from state 2 to state 2
    sigma1 = std. dev. in state 1 (same as std. dev. in state 2 when ix=0)
    sigma2 = std. deviations in state 2 [when ix = 1, omit otherwise]
    rho = ex post probability of state 1 at t=0

*** GLOBAL VARIABLES LISTED ABOVE MUST BE SET BEFORE PROC IS CALLED! ***   */



@ Run this procedure once after y, sameness and vei are defined.
  The syntax is "mkvstart(y);"                                                @

proc(1) = mkvstart(y,vei,nstat); @where vei contains the number of variables@

local i,terms,nrows;
nrows=rows(y);
terms=zeros(2*nstat,2);

if sameness == 0;       @ Explanitory variables differ for all 4 equations @
    terms[1,1]=2;                  terms[1,2]=1+vei[1,1];
i=2;
do while i<= 2*nstat;
terms[i,1]=terms[i-1,2]+1;    terms[i,2]=terms[i,1]+vei[i,1]-1;
i=i+1;
endo;


elseif sameness == 1;   @ Same variables for mean of states 1 and 2 @
  
terms[1:nstat,1]=2*ones(nstat,1);  terms[1:nstat,2]  =(1+vei[1,1])*ones(nstat,1);

i=nstat+1;
do while i<= 2*nstat;
terms[i,1]=terms[i-1,2]+1;    terms[i,2]=terms[i,1]+vei[i,1]-1;
i=i+1;
endo;

elseif sameness == 2;   @ Same variables for prob. of states 1 and 2 @

  terms[1,1]=2;                  terms[1,2]=1+vei[1,1];
i=2;
do while i<= nstat;
terms[i,1]=terms[i-1,2]+1;    terms[i,2]=terms[i,1]+vei[i,1]-1;
i=i+1;
endo;

terms[(nstat+1):(2*nstat),1]=(terms[nstat,2]+1)*ones(nstat,1); terms[(nstat+1):(2*nstat),2]=terms[(nstat+1):(2*nstat),1]+vei[nstat+1,1]-1;


elseif sameness == 3;   @ Same variables for means and same for probs @
  terms[1:nstat,1]=2*ones(nstat,1);  terms[1:nstat,2]  =(1+vei[1,1])*ones(nstat,1);

terms[(nstat+1):(2*nstat),1]=(terms[nstat,2]+1)*ones(nstat,1); terms[(nstat+1):(2*nstat),2]=(terms[nstat+1,1]+vei[nstat+1,1]-1)*ones(nstat,1);

else;                   @ Same variables for all 4 equations @
 
  terms[1:(2*nstat),1]=2*ones(2*nstat,1);  terms[1:(2*nstat),2]=(1+vei[1,1])*ones(2*nstat,1);
endif;
retp(terms);
endp;





proc(1) =swmkv(th,y);


 local  alpha1,alpha2,p,q,sig2,sig1,rho,pxp2,pxp1,
          pxa,pxp,pxpkim,it,qq1,qq2,lf,pth,qth,x,z,nrows;

@ Establish values of parameters @
  alpha1 = th[1:vei[1,1],1];
  alpha2 = th[1+vei[1,1]:vei[1,1]+vei[2,1],1];
  pth = th[1+sumc(vei[1:nstat,1]):sumc(vei[1:nstat,1])+vei[3,1],1];
  qth = th[1+sumc(vei[1:nstat+1,1]):sumc(vei[1:nstat+1,1])+vei[4,1],1];
  sig1 = abs(th[sumc(vei)+1,1]);
  if ix == 0;
    sig2 = sig1;
  else;
    sig2 = abs(th[sumc(vei)+2,1]);
  endif;
  nrows=rows(y);
  lf = zeros(nrows,1);
  pxp=zeros(nrows,1);    
  pxa=zeros(nrows,1);

@ Set the p and q for each period @


 p = cdfn(y[.,terms[nstat+1,1]:terms[nstat+1,2]]*pth);
 q = cdfn(y[.,terms[nstat+2,1]:terms[nstat+2,2]]*qth); 


@ Select the correct value of rho (prob of state 1 at time 0) @
  if iy == 0;             rho = (1-meanc(q))/(2-meanc(p)-meanc(q));
  elseif iy == 1;         rho = cdfn(th[sumc(vei)+2+ix,1]);
  elseif iy == 2;         rho = 1.0;
  elseif iy == 3;         rho = 0;
  endif;
/*
  pxp2 = 1-rho;
  pxp1 = rho;
*/
@ Set PDF's of y for each regime @
  qq1 = pdfn((y[.,1]-y[.,terms[1,1]:terms[1,2]]*alpha1)/sig1)/sig1;
  qq2 = pdfn((y[.,1]-y[.,terms[2,1]:terms[2,2]]*alpha2)/sig2)/sig2;
    @ This allows for underflows in the above expression. @
    if ndpchk(3);   ndpclex;    endif;

@ Start the first loop @ 
/* This loop is the same as mksmooth.g except that we now store the values for pxa1 in a matrix */


if useDLL;
@    dllcall mklz(p,q,qq1,qq2,nrows,rho,lf);@
    dllcall mklz(p,q,qq1,qq2,nrows,rho,lf,pxa,pxp);

else;

    it = 1;     

    do until it > nrows;
      if it == 1;                  @ Step 1 : Ex Ante probability of State 1 @
        pxa[it,1] = (1-q[it,1])*(1-rho) + p[it,1]*rho;
      else;
        pxa[it,1] = (1-q[it,1])*(1-pxp[it-1,1]) + p[it,1]*pxp[it-1,1];
      endif;
      pxp[it,1] = pxa[it,1]*qq1[it,1]; @ Step 2 : Ex Post joint density of State 1 @
                                   @ Step 3 : Likelihood Function @
      lf[it,1] = pxp[it,1]+(1-pxa[it,1])*qq2[it,1];
                                   @ Step 4 : Ex Post probability of State 1 @
      pxp[it,1] = pxp[it,1]/lf[it,1];
      it = it+1;
    endo;

endif;

retp(ln(lf));
endp;


